DROP SCHEMA IF EXISTS `comptaXM`;
CREATE SCHEMA IF NOT EXISTS `comptaXM` DEFAULT CHARACTER SET latin1;
DROP TABLE IF EXISTS `comptaXM`.`previsio`;
DROP TABLE IF EXISTS `comptaXM`.`comanda`;
DROP TABLE IF EXISTS `comptaXM`.`rebut`;
DROP TABLE IF EXISTS `comptaXM`.`periodeComptable`;
DROP TABLE IF EXISTS `comptaXM`.`centre`;
DROP TABLE IF EXISTS `comptaXM`.`usuari`;
DROP TABLE IF EXISTS `comptaXM`.`proveidor`;
DROP TABLE IF EXISTS `comptaXM`.`persona`;

CREATE  TABLE  `comptaXM`.`persona` (
  `idpersona` INT NOT NULL  AUTO_INCREMENT ,
  `document` VARCHAR(10) NOT NULL ,
  `nom` VARCHAR(45) NOT NULL ,
  `cognom1` VARCHAR(45) NULL ,
  `cognom2` VARCHAR(45) NULL ,
  `direccio` VARCHAR(100) NULL ,
  `ciutat` VARCHAR(100) NULL ,
  `telefon` VARCHAR(10) NULL ,
  `email` VARCHAR(100) NULL ,
  `estat` INT(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`idpersona`), 
  UNIQUE INDEX `DOCUMENT_UNIQUE` (`document` ASC))
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `comptaXM`.`usuari` (
  `idusuari` INT NOT NULL ,
  `login` VARCHAR(45) NOT NULL ,
  `password` VARCHAR(45) NOT NULL ,
  `tipusUsuari` varchar(3) NOT NULL DEFAULT 'cap',
  INDEX `fk_usuari_persona_idx` (`idusuari` ASC) ,
  PRIMARY KEY (`idusuari`) ,
  UNIQUE INDEX `login_UNIQUE` (`login` ASC) ,
  CONSTRAINT `fk_usuari_persona`
    FOREIGN KEY (`idusuari` )
    REFERENCES `comptaXM`.`persona` (`idpersona` )
    ON DELETE CASCADE
    ON UPDATE CASCADE)
ENGINE = InnoDB;
CREATE  TABLE IF NOT EXISTS `comptaXM`.`centre` (
  `idcentre` INT NOT NULL ,
  `idUsuari` INT NULL ,
  PRIMARY KEY (`idcentre`) ,
  INDEX `fk_centre_persona_idx` (`idcentre` ASC) ,
  INDEX `fk_centre_usuari_idx` (`idUsuari` ASC) ,
  CONSTRAINT `fk_centre_persona`
    FOREIGN KEY (`idcentre` )
    REFERENCES `comptaXM`.`persona` (`idpersona` )
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT `fk_centre_usuari`
    FOREIGN KEY (`idUsuari` )
    REFERENCES `comptaXM`.`usuari` (`idusuari` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `comptaXM`.`proveidor` (
  `idProveidor` INT(11) NOT NULL AUTO_INCREMENT ,
  `document` VARCHAR(10) NOT NULL ,
  `nom` VARCHAR(45) NOT NULL ,
  `cognom1` VARCHAR(45) NULL ,
  `cognom2` VARCHAR(45) NULL ,
  `direccio` VARCHAR(100) NULL ,
  `ciutat` VARCHAR(100) NULL ,
  `telefon` VARCHAR(10) NULL ,
  `email` VARCHAR(100) NULL ,
  `idCentre` INT(11) NOT NULL ,
  `estat` INT(1) NOT NULL default 1,	
 PRIMARY KEY (`idProveidor`) ,
  INDEX `fk_proveidor_idx` (`idProveidor` ASC) ,
  INDEX `fk_proveidor_centre_idx` (`idCentre` ASC) ,  
  CONSTRAINT `fk_proveidor_centre`
    FOREIGN KEY (`idCentre` )
    REFERENCES `comptaXM`.`centre` (`idcentre` )
    ON DELETE no action
    ON UPDATE no action)
ENGINE = InnoDB;
CREATE  TABLE IF NOT EXISTS `comptaXM`.`periodeComptable` (
  `idperiodeComptable` INT NOT NULL AUTO_INCREMENT ,
  `mes` INT(2) NOT NULL ,
  `anyo` INT(4) NOT NULL ,
  `idcentre` INT NOT NULL ,
  `estat` INT(1) NOT NULL DEFAULT 1 ,
  PRIMARY KEY (`idperiodeComptable`) ,
  INDEX `fk_periode_centre_idx` (`idperiodeComptable` ASC) ,
  CONSTRAINT `fk_periode_centre`
    FOREIGN KEY (`idCentre` )
    REFERENCES `comptaXM`.`centre` (`idcentre` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;
CREATE  TABLE IF NOT EXISTS `comptaXM`.`compte` (
  `idCompte` INT(4) NOT NULL AUTO_INCREMENT ,
  `codi` VARCHAR(8) NOT NULL ,
  `descripcio` VARCHAR(45) NOT NULL ,
  `estat` INT(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`idcompte`) )
ENGINE = InnoDB;
CREATE  TABLE IF NOT EXISTS `comptaXM`.`rebut` (
  `idrebut` INT NOT NULL ,
  `descripcio` VARCHAR(100) NOT NULL ,
  `idperiodeComptable` INT NOT NULL ,
  `valor` DOUBLE(10,2) NOT NULL DEFAULT 0.00 ,
  `idCompte` INT(4) NOT NULL ,
  PRIMARY KEY (`idrebut`) ,
  INDEX `fk_rebut_periode_idx` (`idperiodeComptable` ASC) ,  
  INDEX `fk_rebut_compte_idx` (`idcompte` ASC) ,
  CONSTRAINT `fk_rebut_periode`
    FOREIGN KEY (`idperiodeComptable` )
    REFERENCES `comptaXM`.`periodeComptable` (`idperiodeComptable` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,    
  CONSTRAINT `fk_rebut_compte`
    FOREIGN KEY (`idCompte` )
    REFERENCES `comptaXM`.`compte` (`idCompte` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;

CREATE TABLE `comptaXM`.`previsio` (
  `idprevisio` int(11) NOT NULL,
  PRIMARY KEY (`idprevisio`),
  KEY `fk_previsio_rebut_idx` (`idprevisio`),
  CONSTRAINT `fk_previsio_rebut` FOREIGN KEY (`idprevisio`) REFERENCES `rebut` (`idrebut`) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB ;

CREATE TABLE `comptaXM`.`comanda` (
  `idComanda` int(11) NOT NULL,
  `idProveidor` int(11) DEFAULT NULL,
  `dia` int(2) NOT NULL,
  PRIMARY KEY (`idComanda`),
  KEY `fk_comanda_proveidor_idx` (`idComanda`),
  KEY `fk_comanda_proveidor_idx1` (`idProveidor`),
  CONSTRAINT `fk_comanda_rebut` FOREIGN KEY (`idComanda`) REFERENCES `rebut` (`idrebut`) ON DELETE NO ACTION ON UPDATE NO ACTION,
  CONSTRAINT `fk_comanda_proveidor` FOREIGN KEY (`idProveidor`) REFERENCES `proveidor` (`idProveidor`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB ;

CREATE TABLE `comptaXM`.`periodecomptablecompte` (
  `idPeriodeComptableCompte` int(11) NOT NULL AUTO_INCREMENT,
  `idPeriodeComptable` int(11) NOT NULL,
  `idCompte` int(4) not NULL,
  `totalPrevisio` double(10,2) not NULL default 0.00,
  `totalComanda` double(10,2) not NULL default 0.00,
	PRIMARY KEY (`idPeriodeComptableCompte`),
	CONSTRAINT `fk_periodecomptablecompte_periodeComptable` FOREIGN KEY (`idPeriodeComptable`) REFERENCES `periodeComptable` (`idPeriodeComptable`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	CONSTRAINT `fk_periodecomptablecompte_Compte` FOREIGN KEY (`idCompte`) REFERENCES `compte` (`idCompte`) ON DELETE NO ACTION ON UPDATE CASCADE
) engine=InnoDB ;



delimiter $$
CREATE
DEFINER=`root`@`localhost`
TRIGGER `comptaxm`.`modifica_total_compte_rebut`
AFTER UPDATE ON `comptaxm`.`rebut`
FOR EACH ROW
begin  
declare c long;	
	select count(*) into c from comanda where idComanda=new.idrebut;
	IF(c>0 )THEN
		UPDATE periodecomptablecompte set totalComanda=totalComanda-old.valor 
		WHERE idPeriodeComptable=NEW.idPeriodeComptable and idCompte=old.idCompte;
		select count(*) into c from periodecomptablecompte 
		where idPeriodeComptable=NEW.idPeriodeComptable and idCompte=new.idCompte;
		IF(c>0) THEN 
			UPDATE periodecomptablecompte set totalComanda=totalComanda+new.valor
			WHERE idPeriodeComptable=NEW.idPeriodeComptable and idCompte=new.idCompte;
		ELSE
			INSERT INTO periodecomptablecompte(idperiodecomptable,idcompte,totalComanda)
			VALUES(NEW.idperiodeComptable,NEW.idcompte,new.valor);
		end if;
	end if;
	select count(*) into c from previsio where idPrevisio=new.idrebut;
	IF(c>0 )THEN	
		UPDATE periodecomptablecompte set totalPrevisio=totalPrevisio-OLD.valor
		WHERE idPeriodeComptable=NEW.idPeriodeComptable and idCompte=OLD.idCompte;
		select count(*) into c from periodecomptablecompte 
		where idPeriodeComptable=NEW.idPeriodeComptable and idCompte=new.idCompte;
		IF(c>0) THEN 
			UPDATE periodecomptablecompte set totalPrevisio=totalPrevisio+new.valor
			WHERE idPeriodeComptable=NEW.idPeriodeComptable and idCompte=new.idCompte;
		ELSE
			INSERT INTO periodecomptablecompte(idperiodecomptable,idCompte,totalPrevisio)
			VALUES(NEW.idperiodeComptable,NEW.idCompte,new.valor);
		end if;
	END IF;
end$$


CREATE
DEFINER=`root`@`localhost`
TRIGGER `comptaxm`.`insertar_total_compte_comanda`
AFTER INSERT ON `comptaxm`.`comanda`
FOR EACH ROW
begin  
declare pc int;
declare e int;	
declare compt int;
declare c double;	
	select idPeriodeComptable into pc  from rebut where idrebut=new.idcomanda;
	select idCompte into compt  from rebut where idrebut=new.idcomanda;	
	select count(*) into e From periodecomptablecompte where idPeriodeComptable=pc and idCompte=compt;
	if(e=0) then 	
		INSERT INTO periodecomptablecompte(idPeriodeComptable,idCompte,totalComanda,totalPrevisio)
			values(pc,compt,0,0);		
	end if;
	
	select valor into c from rebut where idRebut=new.idcomanda;			
	UPDATE periodecomptablecompte set totalcomanda=totalcomanda+c
		WHERE idPeriodeComptable=pc and idCompte=compt;	
end$$

CREATE
DEFINER=`root`@`localhost`
TRIGGER `comptaxm`.`borrar_total_compte_comanda`
AFTER DELETE ON `comptaxm`.`comanda`
FOR EACH ROW
begin  
declare pc int;
declare e int;	
declare compt int;
declare c double;	
	select idPeriodeComptable into pc  from rebut where idrebut=old.idcomanda;
	select idCompte into compt  from rebut where idrebut=old.idcomanda;		
	select valor into c from rebut where idRebut=old.idcomanda;			
	UPDATE periodecomptablecompte set totalcomanda=totalcomanda-c
		WHERE idPeriodeComptable=pc and idCompte=compt;	
end$$
CREATE
DEFINER=`root`@`localhost`
TRIGGER `comptaxm`.`insertar_total_compte_previsio`
AFTER INSERT ON `comptaxm`.`previsio`
FOR EACH ROW
begin  
declare pc int;
declare e int;	
declare compt int;
declare c double;	
	select idPeriodeComptable into pc  from rebut where idrebut=new.idprevisio;
	select idCompte into compt  from rebut where idrebut=new.idprevisio;	
	select count(*) into e From periodecomptablecompte where idPeriodeComptable=pc and idCompte=compt;
	if(e=0) then 	
		INSERT INTO periodecomptablecompte(idPeriodeComptable,idCompte,totalComanda,totalPrevisio)
			values(pc,compt,0,0);		
	end if;
	
	select valor into c from rebut where idRebut=new.idprevisio;			
	UPDATE periodecomptablecompte set totalprevisio=totalprevisio+c
		WHERE idPeriodeComptable=pc and idCompte=compt;	
end$$


CREATE
DEFINER=`root`@`localhost`
TRIGGER `comptaxm`.`borra_total_compte_previsio`
AFTER DELETE ON `comptaxm`.`previsio`
FOR EACH ROW
begin  
declare c long;	
declare pc  integer;
declare compt long;
	select valor into c  from rebut where idrebut=old.idprevisio;
	select idPeriodeComptable into pc  from rebut where idrebut=old.idprevisio;
	select idCompte into compt  from rebut where idrebut=old.idprevisio;	

		UPDATE periodecomptablecompte set totalprevisio=totalprevisio-c 
		WHERE idPeriodeComptable=pc and idCompte=compt;

end$$


