package uoc.tfc.xmartimo.model.connexio;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.classic.Session;

import uoc.tfc.xmartimo.bean.connexio.Persona;
import uoc.tfc.xmartimo.bean.connexio.Usuari;

import uoc.tfc.xmartimo.util.HibernateUtil;

/**
 * Classe que representa al model de l'usuari. hereda de HibernateUtil
 * @author xmarti
 *
 */
public class UsuariModel extends HibernateUtil {

	/**
	 * Obt una persona a partir del seu identificador
	 * @param id Identificador intern de la persona
	 * @return Persona	Una persona amb l'identificador entrat
	 */
	public Persona obtenirPersona(int id){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//Query query=session.createQuery("from usuari ");
		Persona persona=new Persona();
		try{
			Query query=session.createQuery("from Persona where idPersona=:id1");
			query.setParameter("id1", id);
			persona = (Persona)query.uniqueResult();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return persona;
	}
	/**
	 * Obt una persona a partir del seu document o identificador extern
	 * @param document	identificacor extern
	 * @return	Persona una persona amb l'identificador extern entrat
	 */
	public Persona obtenirPersona(String document){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//Query query=session.createQuery("from usuari ");
		Persona persona=new Persona();
		try{
			Query query=session.createQuery("from Persona where document=:id1");
			query.setParameter("id1", document);
			persona = (Persona)query.uniqueResult();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return persona;
	}
	/**
	 * Insereix un nou objecte
	 * @param usu
	 */
	public void afegir(Object usu) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		try{
			session.save(usu);
			session.getTransaction().commit();
		}catch (HibernateException he){
			
			session.getTransaction().rollback();					
			
		}
		
	}
	/**
	 * Actualitza un objecte que existeix al sistema
	 * @param usuMod	objecte a modificar
	 * @return	 CERT si l'objecte existeix, FALS en cas contrari
	 */
	public boolean  modificar(Usuari usuMod){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
			
		if(null != usuMod) {
			
			try{
				session.update(usuMod);	
				session.getTransaction().commit();
				
			}catch (HibernateException e){
				System.err.println("Error HIbernate: " +e.getMessage());				
				session.getTransaction().rollback();					
				return false;
			}				
		}
		return true;		
	}
	/**
	 * Obt un usuari a partir del seu identificador intern
	 * @param id	identificador intern
	 * @return	usuari
	 */
	public Usuari getUserById(int id){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		
		Usuari usuari=new Usuari();
		try{
			usuari = (Usuari) session.get(Usuari.class, id);
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			
		}		
		session.getTransaction().commit();
		return usuari;
	}
	/**
	 * Obt un usuari a partir del seu nom d'usuari
	 * @param login	nom d'usuari	
	 * @return	usuari
	 */
	public Usuari getUserByLogin(String login){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//Query query=session.createQuery("from usuari ");
		Usuari usuari=new Usuari();
		try{
			Query query=session.createQuery("from Usuari where login=:log");
			query.setParameter("log", login);
			usuari = (Usuari)query.uniqueResult();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return usuari;
	}
	/**
	 * Obt tots els usuaris del sistema
	 * @return	Llista Usuaris
	 */
	@SuppressWarnings("unchecked")
	public List<Usuari>getUsuaris(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		List<Usuari> usuaris=new ArrayList<Usuari>();
		try{
			Query query=session.createQuery("from Usuari u order by u.login");			
			usuaris = (List<Usuari>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return usuaris;
	}
	/**
	 * Obt una llista d'usuaris a partir del tipus
	 * @param tipus	tipus d'usuari (adm=administrador, cap=capplanta)
	 * @return	llista d'usuaris
	 */
	@SuppressWarnings("unchecked")
	public List<Usuari>getUsuaris(String tipus){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		List<Usuari> usuaris=new ArrayList<Usuari>();
		try{
			Query query=session.createQuery("from Usuari u where u.tipusUsuari=:id order by u.login");
			query.setParameter("id", tipus);
			usuaris = (List<Usuari>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return usuaris;
	}
	/**
	 * Comprova si existeix un altre  usuari a partir del seu identificador extern
	 * @param doc	identificador extern
	 * @param usuari	identificador de l'usuari actual
	 * @return	CERT si existeix, FALS en cas contrari
	 */
	public boolean existeixDocument(String doc, int usuari){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		Persona p= new Persona();
		try{
			Query query=session.createQuery("from Persona p where p.document=:id1 and p.idPersona<>:id2");
			query.setParameter("id1", doc);
			query.setParameter("id2", usuari);
			p = (Persona)query.uniqueResult();
			if (p!=null){return true;}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			return true;
		}		
		session.getTransaction().commit();
		return false;
	}
	/**
	 * Comprova si existeix un altre usuari a partir del seu nom 
	 * @param doc	nom de l'usuari
	 * @param usuari	identificador de l'usuari actual
	 * @return
	 */
	public boolean existeixNom(String doc, int usuari){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		Persona p= new Persona();
		try{
			Query query=session.createQuery("from Persona p where p.nom=:id1 and p.idPersona<>:id2");
			query.setParameter("id1", doc);
			query.setParameter("id2", usuari);
			p = (Persona)query.uniqueResult();
			if (p!=null){return true;}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			return true;
		}		
		session.getTransaction().commit();
		return false;
	}
	/**
	 * Comprova si existeix un altre usuari a partir del seu nom d'usuari
	 * @param login	nom d'usuari
	 * @param usuari	identificador de l'usuari actual
	 * @return	CERT si existeix,FALS en cas contrari
	 */
	public boolean existeixLogin(String login, int usuari){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		Usuari u;
		try{
			Query query=session.createQuery("from Persona p where p.login=:id1 and p.idPersona<>:id2");
			query.setParameter("id1", login);
			query.setParameter("id2", usuari);
			u = (Usuari)query.uniqueResult();
			if (u!=null)return true;
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			return true;
		}		
		session.getTransaction().commit();
		return false;
	}
}
