package uoc.tfc.xmartimo.model.manteniment;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.classic.Session;

import uoc.tfc.xmartimo.bean.connexio.Usuari;
import uoc.tfc.xmartimo.bean.manteniment.Centre;
import uoc.tfc.xmartimo.util.HibernateUtil;
/**
 * Classe que representa al model de dades de centre, Hereda de HIbernateUtil
 * @author Xavier Mart Moret UOC.TFC.J2EE
 *
 */
public class CentreModel extends HibernateUtil {

	/**
	 * Obte tots els centres existents en el sistema
	 * @return LLista de centres
	 */	
	@SuppressWarnings("unchecked")
	public List<Centre> obtenirCentres(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Centre> centres=new ArrayList<Centre>();
		try{
			Query query=session.createQuery("from Centre c order by c.nom asc");			
			centres = (List<Centre>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return centres;
		
	}
	/**
	 * Obt tots els centres actius del sistema
	 * @return	llista de centres
	 */
	@SuppressWarnings("unchecked")
	public List<Centre> obtenirCentresActius(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Centre> centres=new ArrayList<Centre>();
		try{
			Query query=session.createQuery("from Centre c where estat=1 order by c.nom asc");			
			centres = (List<Centre>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return centres;
		
	}
	/**
	 * Obt tots els centres actius que estan associats a un usuari
	 * @param idUsuari	identificador de l'usuari
	 * @return Llista de centres
	 */
	@SuppressWarnings("unchecked")
	public List<Centre> obtenirCentres(int idUsuari){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Centre> centres=new ArrayList<Centre>();
		try{
			Query query=session.createQuery("from Centre where idUsuari=:id and estat=1");
			query.setParameter("id", idUsuari); 
			centres = (List<Centre>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return centres;
	}
	/**
	 * Obt un centre a partir del seu identificador intern
	 * @param id	identificador intern
	 * @return	Centre
	 */
	
	public Centre getCentreById(int id){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		
		Centre centre=new Centre();
		try{
			centre = (Centre) session.get(Centre.class, id);
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			
		}		
		session.getTransaction().commit();
		return centre;
	}
}
