package uoc.tfc.xmartimo.model.manteniment;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.classic.Session;

import uoc.tfc.xmartimo.bean.connexio.Persona;
import uoc.tfc.xmartimo.bean.manteniment.Compte;
import uoc.tfc.xmartimo.util.HibernateUtil;
/**
 * Classe que representa al model de dades de Compte. Hereda de HIbernateUtil
 * @author Xavier Mart Moret UOC.TFC.J2EE
 *
 */
public class CompteModel extends HibernateUtil {
	/**
	 * Obt un compte a partir del seu identificador intern
	 * @param idC	identificador intern
	 * @return	compte
	 */
	public Compte obtenirCompte(int idC){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		Compte compte=new Compte();
		try{
			Query query=session.createQuery("from Compte c where  c.idCompte=:id");			
			query.setParameter("id", idC);
			compte = (Compte)query.uniqueResult();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return compte;
		
	}
	/**
	 * Obt els comptes actius del sistema
	 * @return	Llista de comptes
	 */
	@SuppressWarnings("unchecked")
	public List<Compte> obtenirComptesActius(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Compte> comptes=new ArrayList<Compte>();
		try{
			Query query=session.createQuery("from Compte c where estat=1 order by  c.idCompte asc");			
			
			comptes = (List<Compte>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comptes;		
	}
	/**
	 * Obt una llista de comptes filtrats a partir seu codi i descripci 
	 * @param codi	Filtre de codi
	 * @param descripcio	Filtre de descripci
	 * @return	llista de comtpes
	 */
	@SuppressWarnings("unchecked")
	public List<Compte> obtenirComptes(String codi, String descripcio){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Compte> comptes=new ArrayList<Compte>();
		try{
			Query query=session.createQuery("from Compte c where c.codi LIKE '"+codi+"%' AND c.descripcio LIKE '"+descripcio + "%' order by  c.codi asc");						
			comptes = (List<Compte>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comptes;		
	}
	/**
	 * Comprova si existeix un altre compte a partir del seu codi
	 * @param codi	codi del compte
	 * @param idCompte	identificador del compte actual
	 * @return CERT si existeix, FALS en cas contrari
	 */
	public boolean existeixCompte(String codi, int idCompte){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		Compte p= new Compte();
		try{
			Query query=session.createQuery("from Compte c where c.codi=:id1 and c.idCompte<>:id2");
			query.setParameter("id1", codi);
			query.setParameter("id2", idCompte);
			p = (Compte)query.uniqueResult();
			if (p!=null){return true;}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			return true;
		}		
		session.getTransaction().commit();
		return false;
	}
	
	
}
