package uoc.tfc.xmartimo.model.manteniment;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.classic.Session;
import org.hibernate.criterion.Restrictions;

import uoc.tfc.xmartimo.bean.connexio.Usuari;
import uoc.tfc.xmartimo.bean.manteniment.PeriodeComptable;
import uoc.tfc.xmartimo.bean.manteniment.PeriodeComptableCompte;
import uoc.tfc.xmartimo.util.HibernateUtil;
/**
 * Classe que representa al model de dades del periode comptable. Hereda de HibernateUtil
 * @author Xavier Mart Moret UOC.TFC.J2EE
 *
 */
public class PeriodeComptableModel extends HibernateUtil {
	
	/**
	 * Obt els periodes comptables a partir d'un mes, un any, un centre i un estat
	 * @param idMes mes a cercar
	 * @param idAnyo any a cercar
	 * @param idCentre identificador del centre a cercar
	 * @param estat	estat a cercar
	 * @return llista de perodes comptables
	 */
	@SuppressWarnings("unchecked")
	public List<PeriodeComptable> obtenirPeriodesComptables(int idMes,int idAnyo, int idCentre, int estat){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<PeriodeComptable> periodes=new ArrayList<PeriodeComptable>();
		try{
			String sql="";
			Query query=null;
			if (idMes <=0 && idAnyo<=0 && idCentre <= 0 ){
				if(estat==-1){
					query=session.createQuery("from PeriodeComptable p  order by p.anyo,p.mes,p.centre.nom asc " );
				}else{
					query=session.createQuery("from PeriodeComptable p  where p.estat="+ estat +" order by p.anyo,p.mes,p.centre.nom asc " );
				}
				
			}else{
				if (idMes>0 ){
					sql= " where mes="+idMes;					
				}
				if(idAnyo>0){
					if (sql.equals("")){
						sql=" where anyo="+idAnyo;
					}else{
						sql=sql+ " and anyo=" +idAnyo;
					}					
				}
				if (idCentre> 0){
					if (sql.equals("")){
						sql=" where p.centre.idPersona="+idCentre;
					}else{
						sql=sql+ " and p.centre.idPersona="+idCentre;
					}
				}
				if (estat>=0){
				
						sql=sql+ " and p.estat=" + estat ;					
					}			
				query=session.createQuery("from PeriodeComptable p  "+ sql +"  order by p.anyo,p.mes,p.centre.nom asc " );
			}
						
			periodes = (List<PeriodeComptable>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return periodes;
		
	}
	/**
	 * Obt els periodes comptables a partir d'un mes, un any, un centre, un usuari i un estat
	 * @param idMes mes a cercar
	 * @param idAnyo any a cercar
	 * @param idCentre identificador del centre a cercar
	 * @param idPersona	identificador de l'usuari a cercar
	 * @param estat	estat a cercar
	 * @return	llista de perodes comptables
	 */
	@SuppressWarnings("unchecked")
	public List<PeriodeComptable> obtenirPeriodesComptables(int idMes,int idAnyo, int idCentre,int idPersona, int estat){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<PeriodeComptable> periodes=new ArrayList<PeriodeComptable>();
		try{
			String sql="";
			Query query=null;
			if (idMes <=0 && idAnyo<=0 && idCentre <= 0 ){
				if(estat==-1){
					query=session.createQuery("from PeriodeComptable p where p.centre.usuari.idPersona="+ idPersona+ " order by p.anyo,p.mes,p.centre.nom asc " );
				}else{
					query=session.createQuery("from PeriodeComptable p  where p.centre.usuari.idPersona="+ idPersona+ " and p.estat="+ estat +" order by p.anyo,p.mes,p.centre.nom asc " );
				}
				
			}else{
				if (idMes>0 ){
					sql= " where mes="+idMes;					
				}
				if(idAnyo>0){
					if (sql.equals("")){
						sql=" where anyo="+idAnyo;
					}else{
						sql=sql+ " and anyo=" +idAnyo;
					}					
				}
				if (idCentre> 0){
					if (sql.equals("")){
						sql=" where p.centre.idPersona="+idCentre;
					}else{
						sql=sql+ " and p.centre.idPersona="+idCentre;
					}
				}
				if (estat>=0){
				
						sql=sql+ " and p.estat=" + estat ;					
					}			
				query=session.createQuery("from PeriodeComptable p  "+ sql +" and p.centre.usuari.idPersona="+ idPersona+ " order by p.anyo,p.mes,p.centre.nom asc " );
			}
						
			periodes = (List<PeriodeComptable>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return periodes;
		
	}
	/**
	 * Obt els perodes comptables actius
	 * @return Llista de perodes comptables
	 */
	@SuppressWarnings("unchecked")
	public List<PeriodeComptable> obtenirPeriodesComptablesActius(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<PeriodeComptable> periodes=new ArrayList<PeriodeComptable>();
		try{
			Query query=session.createQuery("from PeriodeComptable p  where estat=1 and p.centre.estat=1 order by p.anyo,p.mes,p.centre.nom asc " );			
			periodes = (List<PeriodeComptable>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return periodes;
		
	}
	/**
	 * Obt els perodes comptables actius que pertanyen a un usuari
	 * @param idUsuari	identificador de l'usuari a cercar
	 * @return	llista de perodes comptables
	 */
	@SuppressWarnings("unchecked")
	public List<PeriodeComptable> obtenirPeriodesComptablesActius(int idUsuari){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<PeriodeComptable> periodes=new ArrayList<PeriodeComptable>();
		try{
			Query query=session.createQuery("from PeriodeComptable p where p.estat=1  and p.centre.estat=1 and p.centre.usuari.idPersona=:id order by p.anyo,p.mes,p.centre.nom asc" );			
			query.setParameter("id", idUsuari);
			periodes = (List<PeriodeComptable>)query.list();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return periodes;
		
	}
	/**
	 * Obt un periode comptable a partir del seu identificador intern
	 * @param idPeriodeComptable identificador intern
	 * @return Periode Comptable
	 */
	public PeriodeComptable obtenirPeriodeComptable(int idPeriodeComptable){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		PeriodeComptable periode=new PeriodeComptable();
		try{
			Query query=session.createQuery("from PeriodeComptable p where  p.idPeriodeComptable=:id");			
			query.setParameter("id", idPeriodeComptable);
			periode = (PeriodeComptable)query.uniqueResult();
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return periode;
		
	}
	@SuppressWarnings("unchecked")
	/**
	 * Obt tots els comptes que estan associtats a un compte.
	 * @param idPeriodeComptable
	 * @return llista de comptes del perode comptable
	 */
	public List<PeriodeComptableCompte> obtenirComptesPerPeriode(int idPeriodeComptable){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<PeriodeComptableCompte> comptesPerPeriode=new ArrayList<PeriodeComptableCompte>();
		try{
			Query query=session.createQuery("from PeriodeComptableCompte r where r.compte.estat=1 and r.periodeComptable.idPeriodeComptable=:id  order by r.compte.idCompte");			
			query.setParameter("id", idPeriodeComptable);
			comptesPerPeriode = (List<PeriodeComptableCompte>)query.list();
			
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comptesPerPeriode;
		
	}
	/**
	 * Obt els comptes d'un periodecomptable a partir d'un any, un centre i un semestre
	 * @param anyo any a cercar
	 * @param centre identificador del centre a cercar
	 * @param semestre semestre a cercar
	 * @return llista de comptes d'un perode comptable
	 */
	@SuppressWarnings("unchecked")
	public List<PeriodeComptableCompte> obtenirComptesPerCentre(int anyo, int centre,int semestre){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();			
		List<Integer>id=new ArrayList<Integer>();
		List<PeriodeComptableCompte> comptesPerPeriode=new ArrayList<PeriodeComptableCompte>();
		try{
			Query query;
			if (semestre==1){
				query=session.createQuery("select r.idPeriodeComptableCompte from PeriodeComptableCompte r where r.periodeComptable.anyo=:id1 and r.periodeComptable.mes>0 and r.periodeComptable.mes<7 and  r.periodeComptable.centre.idPersona=:id2 order by r.compte.codi asc ");
			}else{
				query=session.createQuery("select r.idPeriodeComptableCompte from PeriodeComptableCompte r where r.periodeComptable.anyo=:id1 and r.periodeComptable.mes>6 and r.periodeComptable.mes<13 and  r.periodeComptable.centre.idPersona=:id2 order by r.compte.codi asc ");
			}
			
						
			query.setParameter("id1", anyo);
			query.setParameter("id2", centre);
			id = (List<Integer>)query.list();			
			for(int i:id){
				comptesPerPeriode.add((PeriodeComptableCompte) session.get(PeriodeComptableCompte.class, i));
			}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comptesPerPeriode;
		
	}
	/**
	 * Obt els comptes d'un perode comptable a partir d'un any i d'un mes
	 * @param anyo	any a cercar
	 * @param mes	mes a cercar
	 * @return Llista de comptes d'un periode comptable
	 */
	@SuppressWarnings("unchecked")
	public List<PeriodeComptableCompte> obtenirComptesPerPeriode(int anyo, int mes){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();			
		List<Integer>id=new ArrayList<Integer>();
		List<PeriodeComptableCompte> comptesPerPeriode=new ArrayList<PeriodeComptableCompte>();
		try{
			Query query=session.createQuery("select r.idPeriodeComptableCompte from PeriodeComptableCompte r where r.periodeComptable.anyo=:id1 and r.periodeComptable.mes=:id2 order by r.compte.codi asc ");			
			query.setParameter("id1", anyo);
			query.setParameter("id2", mes);
			id = (List<Integer>)query.list();			
			for(int i:id){
				comptesPerPeriode.add((PeriodeComptableCompte) session.get(PeriodeComptableCompte.class, i));
			}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comptesPerPeriode;
		
	}
	/**
	 * Canvia l'estat d'un periode comptable
	 * @param periode	identificador del periode comptable
	 * @param estat	nou estat del perode comptable
	 * @return 1 si es realitza la transacci, 0 en cas contrari
	 */
	public int updateEstat (int periode, boolean estat){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		int i=0;
		try{
			Query query=session.createQuery("update PeriodeComptable p set p.estat=:id1 where p.idPeriodeComptable=:id2 ");			
			query.setParameter("id1", estat);
			query.setParameter("id2", periode); 
			query.executeUpdate();
			i=1;
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			i=0;
		}		
		session.getTransaction().commit();
		return i;
	}
	/**
	 * Obt els anys els quals existeix un periode comptable
	 * @return	llista d'anys
	 */
	@SuppressWarnings("unchecked")
	public List<Integer> obtenirAnys(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List<Integer>anys= new ArrayList<Integer>();;
		try{
			Query query=session.createQuery("Select distinct pc.anyo from  PeriodeComptable pc order by pc.anyo asc");			
			anys=(List<Integer>)query.list();
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			
		}		
		session.getTransaction().commit();
		return anys;
		
		
	}
}
