package uoc.tfc.xmartimo.model.manteniment;

import java.util.ArrayList;
import java.util.List;



import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.classic.Session;
import uoc.tfc.xmartimo.bean.manteniment.Proveidor;
import uoc.tfc.xmartimo.util.HibernateUtil;
/**
 * Classe que representa el model de dades del provedor. Hereda de HibernateUtil
 * @author Xavier Mart Moret UOC.TFC.J2EE
 *
 */
public class ProveidorModel extends HibernateUtil {
	/**
	 * Obt els proveidors existents en el sistema
	 * @return llista de provedors
	 */
	@SuppressWarnings("unchecked")
	public List<Proveidor> obtenirProveidors(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Proveidor> proveidors=new ArrayList<Proveidor>();
		try{
			Query query=session.createQuery("from Proveidor where estat=1");			
			proveidors = (List<Proveidor>)query.list();
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return proveidors;
		
	}
	/**
	 * Obt els provedors que pertanyen a un centre
	 * @param idCentre	Identificador del centre a cercar
	 * @return	llista de provedors
	 */
	@SuppressWarnings("unchecked")
	public List<Proveidor> obtenirProveidors(int idCentre){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Proveidor> proveidors=new ArrayList<Proveidor>();
		try{
			Query query=session.createQuery("from Proveidor where (idCentre=:id ) and estat=1");
			query.setParameter("id", idCentre); 
			proveidors = (List<Proveidor>)query.list();
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return proveidors;
	}
	/**
	 * Obt els proveidors  a partir de filtrar seu nom i centre
	 * @param nom	nom a filtrar
	 * @param centre identificador del centre a cercar
	 * @return LLista de provedors
	 */
	@SuppressWarnings("unchecked")
	public List<Proveidor> obtenirProveidors(String nom,int centre){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Proveidor> proveidors=new ArrayList<Proveidor>();
		String condicio="";
		try{			
			if(centre<=0){
				condicio=" p.nom LIKE '"+nom+"%' " ;
			}else{
				condicio=" p.nom LIKE '"+nom+"%' and p.centre.idPersona="+centre;
			}											
			Query query=session.createQuery("from Proveidor p where " + condicio +" order by  p.centre.nom, p.cognom1,p.nom asc");
			proveidors = (List<Proveidor>)query.list();
			
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return proveidors;		
	}
	/**
	 * Obt els provedors a partir de filtrar el nom, el centre i l'usuari
	 * @param nom nom a filtrar
	 * @param centre identificador del centre a cercar
	 * @param usuari identificador de l'usuari a cercar
	 * @return Llista de provedors
	 */
	@SuppressWarnings("unchecked")
	public List<Proveidor> obtenirProveidorsCapPlanta(String nom,int centre,int usuari){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Proveidor> proveidors=new ArrayList<Proveidor>();
		String condicio="";
		try{			
			if(centre<=0){
				condicio=" p.centre.idPersona in (select c.idPersona from Centre c where c.usuari.idPersona="+usuari+") and  p.nom LIKE '"+nom+"%' " ;
			}else{
				condicio="p.centre.idPersona in (select c.idPersona from Centre c where c.usuari.idPersona="+usuari+") and  p.nom LIKE '"+nom+"%' and p.centre.idPersona="+centre;
			}			
								
			Query query=session.createQuery("from Proveidor p where " + condicio +" order by  p.centre.nom, p.cognom1,p.nom asc");
			proveidors = (List<Proveidor>)query.list();
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return proveidors;		
	}
	/**
	 * Obt un provedor a partir del seu identificador intern
	 * @param idProveidor Identificador intern
	 * @return provedor
	 */
	public Proveidor obtenirProveidor(int idProveidor){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		Proveidor proveidor=null;
		try{
			Query query=session.createQuery("from Proveidor where idProveidor=:id");
			query.setParameter("id", idProveidor); 
			proveidor = (Proveidor)query.uniqueResult();
			
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return proveidor;
	}
	/**
	 * Comprova si existeix un altre provedor en un mateix centre  partir del seu identificador
	 * @param doc Identificador extern
	 * @param proveidorActual	identificador del proveidor Actual
	 * @param centre	identificador del centre a cercar
	 * @return	provedor
	 */
	public boolean existeixProveidor(String doc, int proveidorActual,int centre){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();		
		//
		Proveidor p= new Proveidor();
		try{
			Query query=session.createQuery("from Proveidor p where p.document=:id1 and p.centre.idPersona=:id2 and p.idProveidor<>:id3)");
			query.setParameter("id1", doc);			
			query.setParameter("id2", centre);
			query.setParameter("id3", proveidorActual);
			p = (Proveidor)query.uniqueResult();
			if (p!=null){return true;}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
			return true;
		}		
		session.getTransaction().commit();
		return false;
	}
}
