package uoc.tfc.xmartimo.model.rebut;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.classic.Session;


import uoc.tfc.xmartimo.bean.connexio.Persona;
import uoc.tfc.xmartimo.bean.manteniment.Proveidor;
import uoc.tfc.xmartimo.bean.rebut.Comanda;
import uoc.tfc.xmartimo.bean.rebut.Previsio;
import uoc.tfc.xmartimo.bean.rebut.Rebut;
import uoc.tfc.xmartimo.util.HibernateUtil;
/**
 * Classe que representa el model de dades del rebut. Hereda de HibernateUtil
 * @author Xavier Mart Moret UOC.TFC.J2EE
 *
 */
public class RebutModel extends HibernateUtil {
	/**
	 * Obt les comandes a partir d'un perode comptable i un compte
	 * @param idPeriodeComptable identificador del perode Comptable a cercar
	 * @param idCompte Identificador del compte a cercar
	 * @return llista de comandes
	 */
	@SuppressWarnings("unchecked")
	public List<Comanda> obtenirComandes(int idPeriodeComptable,int idCompte){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Comanda> comandes=new ArrayList<Comanda>();
		try{
			Query query=session.createQuery("from Comanda as c where c.periodeComptable.idPeriodeComptable=:id1 and c.compte.idCompte=:id2 order by c.dia asc");			
			query.setParameter("id1", idPeriodeComptable);
			query.setParameter("id2", idCompte);
			comandes = (List<Comanda>)query.list();	
			if(comandes!=null){
				for (Comanda c:comandes){
					Proveidor p;
					if(c.getProveidor()!=null){
						Query query1= session.createQuery("from Proveidor where (idProveidor=:id )");
					
						query1.setParameter("id", c.getProveidor().getIdProveidor());					
						p=(Proveidor)query1.uniqueResult();
						c.setProveidor(p);
					}
				}
			}
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comandes;
		
	}
	/**
	 * Obt una comanda a partir del seu identificador
	 * @param idComanda	identificador a cercar
	 * @return comanda
	 */
	public Comanda obtenirComanda(int idComanda){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		Comanda comanda= new Comanda();
		try{
			Query query=session.createQuery("from Comanda as c where c.idRebut=:id1");			
			query.setParameter("id1", idComanda);
			
			comanda = (Comanda)query.uniqueResult();					
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return comanda;
		
	}
	/**
	 * obte l'import total de les comandes d'un periode comptable i un compte
	 * @param idPeriode	identificador del perode comptable a cercar 
	 * @param idCompte	identificador del compte a cercar
	 * @return Double Import de les comandes
	 */
	public double obtenirTotalComandaCompte (int idPeriode,int idCompte){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		double total=0;
		try{
			Query query=session.createQuery("Select p.totalComanda from PeriodeComptableCompte as p where p.periodeComptable.idPeriodeComptable=:id1 and p.compte.idCompte=:id2");			
			query.setParameter("id1", idPeriode);
			query.setParameter("id2", idCompte);			
			total = (double)query.uniqueResult();					
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return total;
	}
	/**
	 * Obte les previsions d'un perode comptable i un compte
	 * @param idPeriodeComptable	Identificador del periode Comptable  a cercar
	 * @param idCompte	identificador del compte  a cercar
	 * @return	Llista de previsions
	 */
	@SuppressWarnings("unchecked")
	public List<Previsio> obtenirPrevisions(int idPeriodeComptable,int idCompte){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		List<Previsio> previsions=new ArrayList<Previsio>();
		try{
			Query query=session.createQuery("from Previsio as p where p.periodeComptable.idPeriodeComptable=:id1 and p.compte.idCompte=:id2 order by p.descripcio asc");			
			query.setParameter("id1", idPeriodeComptable);
			query.setParameter("id2", idCompte);
			previsions = (List<Previsio>)query.list();					
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return previsions;
		
	}
	/**
	 * Obte una previsi a partir del seu identificador
	 * @param idPrevisio	identificador de la previsi a cercar
	 * @return	previsio
	 */
	public Previsio obtenirPrevisio(int idPrevisio){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		Previsio previsio= new Previsio();
		try{
			Query query=session.createQuery("from Previsio as c where c.idRebut=:id1");			
			query.setParameter("id1", idPrevisio);
			
			previsio = (Previsio)query.uniqueResult();					
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return previsio;
	}
	/**
	 * Obt l'import total de les previsions que pertanyen a un perode comptable i un compte
	 * @param idPeriode	identificador del perode comptable
	 * @param idCompte identificador del compte
	 * @return	Double import total de les previsions
	 */
	public double obtenirTotalPrevisioCompte (int idPeriode,int idCompte){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();				
		double total=0;
		try{
			Query query=session.createQuery("Select p.totalPrevisio from PeriodeComptableCompte as p where p.periodeComptable.idPeriodeComptable=:id1 and p.compte.idCompte=:id2");			
			query.setParameter("id1", idPeriode);
			query.setParameter("id2", idCompte);			
			total = (double)query.uniqueResult();					
		}catch (HibernateException he){
			System.err.println("Error Hibernate: " + he.getMessage());
			session.getTransaction().rollback();
		}		
		session.getTransaction().commit();
		return total;
	}
}
