package uoc.tfc.xmartimo.util;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.classic.Session;

/**
 * Classe que representa a les utilitats imprescindibles per a portar a terme les transaccions a la base de dades<br>
 * Tamb utititats per realitzar tasques comunes d'inserci, modificaci i eliminaci.
 * @author Xavier Mart Moret UOC.TFC.J2EE
 *
 */
public class HibernateUtil {

	private static final SessionFactory sessionFactory = buildSessionFactory();
	/**
	 * Crea una sessi per hibernate a partir dels parmetres del fitxer de configuraci
	 * @return
	 */
	private static SessionFactory buildSessionFactory() {
		try {
			// Create the SessionFactory from hibernate.cfg.xml
			return new AnnotationConfiguration().configure()
					.buildSessionFactory();
		} catch (Throwable ex) {
			System.err.println("Initial SessionFactory creation failed." + ex);
			throw new ExceptionInInitializerError(ex);
		}
	}
	/**
	 * Obt una sessi 
	 * @return SessionFactory
	 */
	public static SessionFactory getSessionFactory() {
		return sessionFactory;
	}
	/**
	 * Insereix un objecte a la base de dades
	 * @param object Objecte a inserir
	 * @return CERT si es realitza l'operaci, FALS en cas contrari
	 */
	public boolean add(Object object) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		try{
			session.save(object);
			session.getTransaction().commit();
		}catch (Exception e){			
			session.getTransaction().rollback();
			System.out.println(e.getClass()+" " +e.getCause()+ " "+e.getMessage());
			return false;
		}
		
		return true;
	}
	/**
	 * Elimina un objecte de la base de dades
	 * @param object objecte a eliminar
	 */
	public void delete(Object object){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		session.delete(object);
		session.getTransaction().commit();
	}
	/**
	 * Modifica un objecte de la base de dades
	 * @param object Objecte a modificar
	 * @return CERT si es realtiza l'operaci, FALS en cas contrari
	 */
	public boolean  update(Object object){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		try{
			session.update(object);
			
			session.getTransaction().commit();
		}catch (Exception e){
			session.getTransaction().rollback();
			System.out.println(e.getClass()+" " +e.getCause()+ " "+e.getMessage());
			return false;
			
		}
		
		
		return true;
	}
}
